require_dependency 'issues_controller'

module InlineEditingIssuesControllerPatch
  

    def self.included(base) # :nodoc:
      
      base.send(:include, InstanceMethods)
      base.class_eval do
        unloadable
        before_filter :find_issue, :only => [:show, :edit, :update, :show_status_list, :change_issue_status, 
                                             :show_tracker_list, :change_issue_tracker,
                                             :show_priority_list, :change_issue_priority,
                                             :show_assignee_list, :change_issue_assignee,
                                             :show_fixed_version_list, :change_issue_fixed_version,
                                             :show_due_date_field, :change_issue_due_date,
                                             :show_estimated_hours_field, :change_issue_estimated_hours,
                                             :show_spent_hours_field, :change_issue_spent_hours]
        before_filter :find_project, :only => [:new, :create, :show_status_list, :change_issue_status, 
                                               :show_tracker_list, :change_issue_tracker,
                                               :show_priority_list, :change_issue_priority,
                                               :show_assignee_list, :change_issue_assignee,
                                               :show_fixed_version_list, :change_issue_fixed_version,
                                               :show_due_date_field, :change_issue_due_date,
                                               :show_estimated_hours_field, :change_issue_estimated_hours,
                                               :show_spent_hours_field, :change_issue_spent_hours]
        before_filter :init_journal, :only => [:change_issue_status, :change_issue_tracker, :change_issue_priority,
                                               :change_issue_assignee, :change_issue_fixed_version, :change_issue_due_date,
                                               :change_issue_estimated_hours]
      end
        
    end
   
    module InstanceMethods
      
      def show_status_list
        @status = @issue.status
        @allowed_statuses = @issue.new_statuses_allowed_to(User.current)
        render :partial => 'issues/show_issue_statuses', :layout => false, :locals => {:issue => @issue, :statuses => @allowed_statuses}
      end
  
      def change_issue_status
        @issue.update_attributes(:status_id => params[:status][:id])
        render :partial => 'issues/change_issue_status', :layout => false, :locals => {:issue => @issue}
      end
      
      def show_tracker_list
        @tracker = @issue.tracker
        @allowed_trackers = @issue.project.trackers
        render :partial => 'issues/show_issue_trackers', :layout => false, :locals => {:issue => @issue, :trackers => @allowed_trackers}
      end
  
      def change_issue_tracker
        @issue.update_attributes(:tracker_id => params[:tracker][:id])
        render :partial => 'issues/change_issue_tracker', :layout => false, :locals => {:issue => @issue}
      end
      
      def show_priority_list
        @priority = @issue.priority
        @allowed_priorities = IssuePriority.find_all_by_active(true)
        render :partial => 'issues/show_issue_priorities', :layout => false, :locals => {:issue => @issue, :priorities => @allowed_priorities}
      end
  
      def change_issue_priority
        @issue.update_attributes(:priority_id => params[:priority][:id])
        render :partial => 'issues/change_issue_priority', :layout => false, :locals => {:issue => @issue}
      end
      
      def show_assignee_list
        @assignee = @issue.assigned_to
        @assignable_users = @issue.assignable_users
        render :partial => 'issues/show_issue_assignee', :layout => false, :locals => {:issue => @issue, :assignable_users => @assignable_users}
      end
  
      def change_issue_assignee
        @issue.update_attributes(:assigned_to_id => params[:assignee][:id])
        render :partial => 'issues/change_issue_assignee', :layout => false, :locals => {:issue => @issue}
      end
      
      def show_fixed_version_list
        @fixed_version = @issue.fixed_version
        @assignable_versions = @issue.assignable_versions
        render :partial => 'issues/show_issue_fixed_version', :layout => false, :locals => {:issue => @issue, :assignable_versions => @assignable_versions}
      end
  
      def change_issue_fixed_version
        @issue.update_attributes(:fixed_version_id => params[:fixed_version][:id])
        render :partial => 'issues/change_issue_fixed_version', :layout => false, :locals => {:issue => @issue}
      end
      
      def show_due_date_field
        render :partial => 'issues/show_due_date_field', :layout => false, :locals => {:issue => @issue}
      end
  
      def change_issue_due_date
        @issue.update_attributes(:due_date => params[:issue][:due_date])
        render :partial => 'issues/change_issue_due_date', :layout => false, :locals => {:issue => @issue}
      end
      
      def show_estimated_hours_field
        render :partial => 'issues/show_estimated_hours_field', :layout => false, :locals => {:issue => @issue}
      end
  
      def change_issue_estimated_hours
        @issue.update_attributes(:estimated_hours => params[:issue][:estimated_hours]) if @issue.leaf?
        parent_issue = @issue.parent
        render :update do |page|
          page.replace_html "#{@issue.id}-estimated_hours".to_sym, :partial => 'issues/change_issue_estimated_hours', :locals => {:issue => @issue}
          page.replace_html "#{parent_issue.id}-estimated_hours".to_sym, :partial => 'issues/change_issue_estimated_hours', :locals => {:issue => parent_issue} if parent_issue
        end
      end
      
      def show_spent_hours_field
        @time_entry_activities = TimeEntryActivity.find_all_by_active(true)
        @time_entry = TimeEntry.new(:issue => @issue, :project => @issue.project)
        render :partial => 'issues/show_spent_hours_field', :layout => false, :locals => {:issue => @issue, :time_entry_activities => @time_entry_activities}
      end
  
      def change_issue_spent_hours
        @time_entry = TimeEntry.new(:issue => @issue, :project => @issue.project, :user => User.current, :spent_on => Date.today)
        @time_entry.hours = params[:time_entry][:hours]
        @time_entry.activity_id = params[:time_entry][:activity_id]
        if @time_entry.save
          @issue.init_journal(User.current, notes = "Spent time increased by #{sprintf("%.2f", @time_entry.hours)} hours")
          @issue.touch
        end
        render :partial => 'issues/change_issue_spent_hours', :layout => false, :locals => {:issue => @issue}
      end
      
      private
      
      def init_journal
        @issue.init_journal(User.current, notes = "")
      end
      
    end
    
end
 
IssuesController.send(:include, InlineEditingIssuesControllerPatch)
 
